/*****************************************************************************
 * commands.c: lirc commands handling
 *****************************************************************************
 * $Id: commands.c,v 1.36 2004/04/17 22:36:40 pingus77 Exp $
 *****************************************************************************
 * Copyright (C) 2001 Keuleu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************
 *
 * This software was based on xawtv. Those portions are
 * Copyright (C) 2000 Gerd Knorr <kraxel@cs.tu-berlin.de>
 *
 *****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <math.h>
#include <stdarg.h>
#include <time.h>

#include <X11/Intrinsic.h>

#include "config.h"
#include "commands.h"
#include "channel.h"
#include "plugin.h"

/* ----------------------------------------------------------------------- */
extern int debug;
extern int res_mode;
extern char *res_modes[];
extern Widget app_shell;
extern PLUGIN *plugin;

void SetChannelAction (Widget, XEvent *, String *, Cardinal *);
void SetSourceAction (Widget, XEvent *, String *, Cardinal *);
void VolumeAction (Widget, XEvent *, String *, Cardinal *);
void FullScreenAction (Widget, XEvent *, String *, Cardinal *);
void SetResAction (Widget, XEvent *, String *, Cardinal *);
void RecordMovieAction (Widget, XEvent *, String *, Cardinal *);
void ShowtimeAction (Widget, XEvent *, String *, Cardinal *);
void SubtitlesPagenoAction (Widget, XEvent *, String *, Cardinal *);
void UnSubtitlesPagenoAction (Widget, XEvent *, String *, Cardinal *);
void ToggleSubAction (Widget, XEvent *, String *, Cardinal *);
void SnapAction (Widget, XEvent *, String *, Cardinal *);
void AleVTAction (Widget, XEvent *, String *, Cardinal *);
void ExitCB (Widget widget, XtPointer client_data, XtPointer calldata);
void osd (int x_osd, int y_osd, char *osd_msg, int osd_time);
void TuneAction (Widget, XEvent *, String *, Cardinal *);
void DeinterlaceAction (Widget, XEvent *, String *, Cardinal *);
void ZapAction (Widget, XEvent *, String *, Cardinal *);
#ifdef USE_LIBXOSD
void XosdAction();
#endif


char *commands_list[] = {
"setstation",		"Set the TV station",
"setchannel",		"Tune TV channel",
"deinterlace",		"Select a deinterlace filter",
"zap",			"Channel Hopping Slow / Fast / Off",
"volume",		"Set the volume",
"fullscreen",		"Toggle fullscreen mode",
"setres",		"Set resource value",
"showtime",		"Display time",
"xosd",			"Toggles xosd use for osd display",
"list",			"Lists all xawdecode commands",
"quit",			"Quit xawdecode",
"keypad",		"Enter digit n",
"record",		"Starts/stops recording",
"subpageno",		"Display subtitles: Choose subtitles page number",
"togglesub",		"Toggles subtitles: Activate/Deactivate subtitles",
"teletext",		"Starts/stops Videotext/Teletext decoder",
"vtx",			"Display subtitles",
"run",			"Runs an external command",
"setinput",		"Sets the input source (eg. Television, Camera, S-VIDEO, etc)",
"snap"			"capture an image",
(char*)-1,			(char*)-1
};

/* ----------------------------------------------------------------------- */

int get_arg(char **arg)
{
  int n;
  char *tok;

  if ((tok=strtok(NULL, " ")) != NULL)
    n = 1;
  else
    n = 0;

  *arg = tok;

  return n;
}

int
do_command_single(char *lirc_cmd)
{
    char *tok;
    int   argc, i=0;
    char *cmd;

    /* strtok should always be used in the place of strtok
       but in this case, it should also be changed in the plugins....
       SO be sure to use strtok in all the codes run by this function
       (in particular avilib.c and alevt_main.c)
    */

    cmd = malloc(strlen(lirc_cmd)+1);
    strcpy(cmd, lirc_cmd);

    tok = strtok(cmd, " ");
    if(tok != NULL)
      {
	if (strcmp(tok, "setstation") == 0)
          {
	    argc = get_arg(&tok); // next, prev
	    SetChannelAction(NULL, NULL, &tok, &argc);
          }
	else if (strcmp(tok, "setchannel") == 0)
          {
	    argc = get_arg(&tok); // next, prev
	    TuneAction(NULL, NULL, &tok, &argc);
          }
	else if (strcmp(tok, "deinterlace") == 0)
          {
	    argc = 0; // no arg just a list
	    DeinterlaceAction(NULL, NULL, NULL, &argc);
          }
	else if (strcmp(tok, "zap") == 0)
          {
	    argc = get_arg(&tok); // no argument = Slow Hopping / Off; argument = fast => Fast Hopping
	    ZapAction(NULL, NULL, &tok, &argc);
          }
	else if (strcmp(tok, "resmode") == 0)
          {
	    res_mode = (res_mode + 1) % 4;
	    osd(30, 20, res_modes[res_mode], 6000);
          }
	else if (strcmp(tok, "resource") == 0)
          {
            char *argv[2];
            argc = get_arg(&tok); // numeric value i.e.: resource +640 or resource -640 only for LIRC!!
            argv[1] = tok;
            argc += 1;
            argv[0] = res_modes[res_mode];
            SetResAction (NULL, NULL, argv, &argc);
          }
	else if  (strcmp(tok, "volume") == 0)
          {
            argc = get_arg(&tok); // inc, dec, mute or numeric value
            VolumeAction(NULL, NULL, &tok, &argc);
          }
        else if  (strcmp(tok, "fullscreen") == 0)
          {
            argc = get_arg(&tok); // toggle
            FullScreenAction(NULL, NULL, &tok, &argc);
          }
        else if  (strcmp(tok, "setres") == 0)
          {
            char *argv[2];
            argc = get_arg(&tok); // capture
            argv[0] = tok;
            argc += get_arg(&tok); // toggle
            argv[1] = tok;
            SetResAction (NULL, NULL, argv, &argc);
          }
        else if  (strcmp(tok, "showtime") == 0)
          {
            // no args
            argc = 0;
            ShowtimeAction (NULL, NULL, NULL, NULL);
          }
#ifdef USE_LIBXOSD
	 else if  (strcmp(tok, "xosd") == 0)
          {
            // no args
            argc = 0;
            XosdAction ();
          }
#endif
        else if  (strcmp(tok, "list") == 0)
		{
			printf("Supported xawdecode commands:\n");
			while ((char *)-1 != commands_list[i])
			{
				printf("%s\t", commands_list[i]);
				printf("  \t%s\n", commands_list[++i]);
				i++;
			}
		}
		else if  (strcmp(tok, "run") == 0)
		{
			char command[255] = "\0";
			int argc;
			for (i=0;i < 9;i++)
			{
				argc += get_arg(&tok);
				if (tok)
				{
					strcat (command, tok);
					strcat (command, " ");
				}
			}
			printf("Running %s\n", command);
			system(command);
		}
		else if  (strcmp(tok, "quit") == 0)
          {
            // no args
            argc = 0;
            ExitCB (app_shell, NULL, NULL);
          }
        else if  (strcmp(tok, "keypad") == 0)
          {
            argc = get_arg(&tok); // [0..9]
	    if (NULL != tok)
    		SetChannelAction(NULL, NULL, &tok, &argc);
          }
        else if  (strcmp(tok, "record") == 0)
          {
            // no args
            argc = 0;
            RecordMovieAction(NULL, NULL, NULL, &argc);
          }
        else if  (strcmp(tok, "subpageno") == 0)
          {
            // no args
            argc = 0;
            SubtitlesPagenoAction(NULL, NULL, NULL, &argc);
          }
        else if  (strcmp(tok, "unsubpageno") == 0)
          {
            // no args
            argc = 0;
            SubtitlesPagenoAction(NULL, NULL, NULL, &argc);
          }
        else if  (strcmp(tok, "togglesub") == 0)
          {
            // no args
            argc = 0;
            ToggleSubAction(NULL, NULL, NULL, &argc);
          }
        else if  (strcmp(tok, "teletext") == 0)
          {
            // no args
            argc = 0;
            AleVTAction(NULL, NULL, NULL, &argc);
          }
	else if (strcmp(tok, "setinput") == 0)
	  {
	    argc = get_arg(&tok); // next, prev
            SetSourceAction(NULL, NULL, &tok, &argc);
	  }
        else if  (strcmp(tok, "snap") == 0)
          {
            char *argv[2];
            argc = get_arg(&tok); // file format
            argv[0] = tok;
	    if (debug)
	      fprintf(stderr,"First Argument: %s\n",tok);
            argc += get_arg(&tok); // full/win
            argv[1] = tok;
	    if (debug)
	      fprintf(stderr,"Second Argument: %s\n",tok);
            SnapAction(NULL, NULL, argv, &argc);
          }
        else if  (strcmp(tok, "vtx") == 0)
          {
	    int i,j,nblines=0;
	    tok += strlen(tok)+1;
	    if(tok==cmd+strlen(lirc_cmd)+1)
	      tok="";
	    else {
	      for(i=0;i<strlen(tok);i++) if(tok[i]=='|') tok[i]='\n';
	      /* remove the first empty lines...*/
	      for(i=0,j=-1;tok[i]==' '||tok[i]=='\n';i++)
		if(tok[i]=='\n') j=i;
	      tok=tok+j+1;
	    }
	    for(i=0;i<strlen(tok);i++) if(tok[i]=='\n') nblines++;
	    if(nblines>8) {
	      if(debug) {
		fprintf(stderr,"teletext page contains %d lines\n"
			" does not seem to be a subtitle\n",nblines);
	      }
	    } else
	      osd(-1, -1, tok, -1);
	    tok=NULL;
          }
        else
          {
            int ret = -1;
            if (plugin)
              ret = plugin->do_command(&tok);
            if (ret == -1)
              fprintf (stderr, "lirc: unknown command: %s\n", tok);
          }
      }
    if(tok!=NULL) tok=strtok(NULL, " ");
    if(tok!=NULL)
      fprintf (stderr,"lirc: not all the arguments are used\n");

    free(cmd);

    return 0;
}

void do_command(char *c) {
  char *tok,*s;

  if(c==NULL) return;
  c=strdup(c);
  tok= strtok_r(c, ";",&s);
  while(tok!=NULL) {
    do_command_single(tok);
    tok=strtok_r(NULL,";",&s);
  }
  free(c);
}

char**
split_cmdline(char *line, int *count)
{
    static char cmdline[1024];
    static char *argv[32];
    int  argc,i;

    strcpy(cmdline,line);
    for (argc=0, i=0; argc<31;) {
	argv[argc++] = cmdline+i;
	while (cmdline[i] != ' ' &&
	       cmdline[i] != '\t' &&
	       cmdline[i] != '\0')
	    i++;
	if (cmdline[i] == '\0')
	    break;
	cmdline[i++] = '\0';
	while (cmdline[i] == ' ' ||
	       cmdline[i] == '\t')
	    i++;
	if (cmdline[i] == '\0')
	    break;
    }
    argv[argc] = NULL;

    *count = argc;
    return argv;
}
