/*****************************************************************************
 * grab-bktr.c: bktr interface (OpenBSD/NetBSD/FreeBSD drivers)
 *****************************************************************************
 * $Id: grab-bktr.c,v 1.11 2004/04/26 20:01:26 alainjj Exp $
 *****************************************************************************
 * Copyright (C) 2004 Alainjj
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************
 *
 * for the moment only a slow grabdisplay is provided
 * inspired from:
 * xawtv/libng/plugins/drv0-bsd.c
 * fxtv/tvcapture.c
 * mplayer/libmpdemux/tvi_bsdbt848.c 
 * http://www.gsp.com/cgi-bin/man.cgi?section=4&topic=meteor
 *
 * if you have some problems try with "-noxv" option first
 *
*****************************************************************************/


#include "config.h"
#ifdef HAVE_BKTR

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/filio.h>
#include <sys/time.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <pthread.h>

#if HAVE_DEV_IC_BT8XX_H
#include <dev/ic/bt8xx.h>
#else
#include <machine/ioctl_meteor.h>
#include <machine/ioctl_bt848.h>
#endif

#include "strtab.h"
#include "grab.h"
#include "memcpy.h"
#include "colorspace.h"
#include <X11/Intrinsic.h>
#include "channel.h"
#include "plugin.h"

extern int bttvfd;
static int tunerfd = -1;
extern int vbifd, nbufs_default, nbufs, img;
extern struct GRABBER grab_dummy;
char *buf=NULL;
int bktr_swap=0, bktr_yuvbug=0, bktr_mode=-1, bktr_read=0;
static int sizeframe,sizeframe2;
extern PLUGIN *plugin;

/* prototypes */
static int grab_open (struct device_t *device, int sw, int sh,
                      int format, int pixmap, void *base, int width);
static int grab_close ();
static int grab_overlay (int x, int y, int width, int height, int format,
                         struct OVERLAY_CLIP *oc, int count);
static void *grab_scr (void *dest, int width, int height, int single,
                       int dest_xsize);
static void *grab_one (int format, int width, int height);
static int grab_tune (unsigned long freq);
static int grab_tuned ();
static int grab_input (int input, int norm);
static int grab_picture (int color, int bright, int hue, int contrast);
static int grab_audio (int mute, int volume, int *mode);
static int is525(int norm);
static int issecam(int norm);
static void* get_buf(int i);

static struct STRTAB norms[] = {
    {  0, "NTSC"      },
    {  1, "NTSC-JP"   },
    {  2, "PAL"       },
    {  3, "PAL-M"     },
    {  4, "PAL-N"     },
    {  5, "SECAM"     },
    {  6, "RSVD"      },
    { -1, NULL }
};
static int norms_map[] = {
    BT848_IFORM_F_NTSCM,
    BT848_IFORM_F_NTSCJ,
    BT848_IFORM_F_PALBDGHI,
    BT848_IFORM_F_PALM,
    BT848_IFORM_F_PALN,
    BT848_IFORM_F_SECAM,
    BT848_IFORM_F_RSVD,
};
static int norms_map2[] = {
  METEOR_FMT_NTSC,
  METEOR_FMT_NTSC,
  METEOR_FMT_PAL,
  METEOR_FMT_PAL,
  METEOR_FMT_PAL,
  METEOR_FMT_SECAM,
  METEOR_FMT_AUTOMODE /* ? */
};

static struct STRTAB inputs[] = {
    {  0, "Television"   },
    {  1, "Composite1"   },
    {  2, "S-Video"      },
    {  3, "CSVIDEO"      },
    { -1, NULL }
};
static int inputs_map[] = {
    METEOR_INPUT_DEV1,
    METEOR_INPUT_DEV0,
    METEOR_INPUT_DEV_SVIDEO,
    METEOR_INPUT_DEV2,
};

struct GRABBER grab_bktr = {
  "bktr",
  VIDEO_RGB16 | VIDEO_RGB24 | VIDEO_RGB32,
  0,
  norms, inputs,
  grab_open,
  grab_close,
  grab_overlay,
  grab_scr,
  grab_one,
  grab_tune,
  grab_tuned,
  grab_input,
  grab_picture,
  grab_audio,
  is525,
  issecam,
  get_buf
};

static char *tvfilename="/dev/bktr0", *tunerfilename="/dev/tuner0",
  *vbifilename="/dev/vbi0";

static struct meteor_pixfmt fmt[64], *cur_fmt;
int cur_f;

extern int x11_bigendian;
static struct meteor_pixfmt * xaw_to_fmt(int f) {
  int i;
  if(bktr_mode!=-1) return &fmt[bktr_mode];
  for(i=0;i<64;i++) {
    int b=0;
    if(fmt[i].index==-1) continue;
    switch(fmt[i].type) {
    case METEOR_PIXTYPE_RGB:
      switch(f) {
      case VIDEO_RGB15:
	b = fmt[i].masks[0]==(31<<10) && 
	  fmt[i].swap_bytes==!x11_bigendian;
	break;
      case VIDEO_RGB16:
	b = fmt[i].masks[0]==(31<<11) && 
	  fmt[i].swap_bytes==!x11_bigendian;
	break;
      case VIDEO_RGB24:
	b = fmt[i].masks[0]==(255<<16) && fmt[i].Bpp==3;
	break;
      case VIDEO_RGB32:
	b = fmt[i].masks[0]==(255<<16) && fmt[i].Bpp==4 
	  && fmt[i].swap_shorts==!x11_bigendian
	  && fmt[i].swap_bytes==!x11_bigendian;
	break;
      case VIDEO_YUY2:
	if(bktr_yuvbug)
	  b = fmt[i].masks[0]==(255<<16) && fmt[i].Bpp==3; 
	break;
      }
      break;
    case METEOR_PIXTYPE_YUV_PACKED: 
      b = f==VIDEO_YUY2;
      break;
    default:
    }
    if(b) {
      printf("@@@@@@@@@@ f=%d i=%d\n",f,i); return &fmt[i];
    }
  }
  return NULL;
}

static int xaw_to_geo(int f) {
  switch(f) {
  case VIDEO_RGB15:
  case VIDEO_RGB16:
    return METEOR_GEO_RGB16;
  case VIDEO_RGB24:
  case VIDEO_RGB32:
    return METEOR_GEO_RGB24;
  case VIDEO_YUY2:
    return METEOR_GEO_YUV_PACKED;
  default:
    return 0;
  }
}


void dummy(int signal) {
  if(debug>3)
    fprintf(stderr,"SIGUSR1\n");
}

static int
grab_open(struct device_t *device, int sw, int sh,
          int format, int pixmap, void *base, int bpl)
{
  int i;
  
  if (-1 == (bttvfd = open (tvfilename, O_RDONLY))) {
    fprintf (stderr, "open %s: %s\n", tvfilename, strerror (errno));
    goto err;
  }
  if (-1 == (tunerfd = open (tunerfilename, O_RDONLY))) {
    fprintf (stderr, "open %s: %s\n", tunerfilename, strerror (errno));
    goto err;
  }
  if(device->vbi) /* just for the -novbi option */
    if (-1 == (vbifd = open (vbifilename, O_RDONLY))) {
      fprintf (stderr, "bktr: cant't open %s for reading, %s\n",
	       vbifilename, strerror (errno));
    }
  fprintf(stderr,"supported formats\n");
  for(i=0;i<64;i++) {
    fmt[i].index=i;
    if(-1 == ioctl(bttvfd, METEORGSUPPIXFMT, &fmt[i]))
      fmt[i].index=-1;
    else {
      struct meteor_pixfmt *a=&fmt[i];
      fprintf(stderr,"(%d,type=%d,bpp=%d,"
	      "masks=0x%lx/0x%lx/0x%lx,sb=%d,ss=%d)\n",
	      a->index,a->type,a->Bpp,a->masks[0],a->masks[1],a->masks[2],
	      a->swap_bytes,a->swap_shorts);
    }
  }
  cur_f=pixmap;
  cur_fmt=xaw_to_fmt(pixmap);
  if(cur_fmt==NULL) {
    fprintf(stderr,"format not available by card\n");
    goto err;
  }
  signal(SIGUSR1, dummy);
  img=0;
  if(plugin) nbufs=nbufs_default; else nbufs=0;
  return 0;
 err:
  grab_close();
  return -1;
}

static int
grab_close ()
{
  if(tunerfd != -1) {
    grab_audio (1, -1, NULL);
    close(tunerfd);
    tunerfd = -1;
  }
  if(bttvfd != -1) {
    close(bttvfd);
    bttvfd = -1;
  }
  if(vbifd != -1) {
    close(vbifd);
    vbifd = -1;
  }
  return 0;
}


static int
grab_overlay (int x, int y, int width, int height, int format,
              struct OVERLAY_CLIP *oc, int count)
{
  int arg;
  printf("grab_ov\n");
  if(bttvfd==-1) return -1;
  if (width == 0 || height == 0) {
    if (debug) fprintf (stderr, "bktr: overlay off\n");
    arg=0;
    if( -1 == ioctl(bttvfd, METEORCAPTUR, &arg ))
      perror("ioctl METEORCAPTUR off");
      return 0;
  }
  return 0;
}

static void rgb24_to_yuv(void *dest, void *src, int size) {
  unsigned char *p1=src, *p2=dest;
  int i;
  for(i=0;i<size;i+=6,p1+=6,p2+=4) {
    unsigned char r,g,b;
    b=p1[0]; g=p1[1]; r=p1[2];
    p2[0] = ((16829*r+31913*g+6416*b)>>16)+16;
    p2[1] = ((-9750*r-19148*g+28898*b)>>16)+128;
    b=p1[3]; g=p1[4]; r=p1[5];
    p2[2] = ((16829*r+31913*g+6416*b)>>16)+16;
    p2[3] = ((28898*r-24199*g-4699*b)>>16)+128;
  }
}

void *bktr_getframe(void *dest,int width, int height) {
  static struct meteor_geomet geom;
  static int firsttime=1;
  struct timeval tv;
  sigset_t sa_mask;
  static void *map=NULL;
  static int last_f=-1;
  
  if(firsttime || width!=geom.columns || height!=geom.rows || last_f!=cur_f) {
    int arg;
    last_f=cur_f;
    if(!firsttime && !bktr_read) {
      arg=METEOR_CAP_STOP_CONT;
      if(ioctl(bttvfd, METEORCAPTUR, &arg)<0) {
	perror("ioctl METEORCAPTUR STOP_CONT");
	return NULL;
      }
      arg=METEOR_SIG_MODE_MASK;
      if(-1 == ioctl(bttvfd, METEORSSIGNAL, &arg))
	perror("ioctl METEORSSIGNAL off");
      if(!bktr_read && -1 == munmap(map, sizeframe)) {
	perror("munmap");
      }
    }
    firsttime=0;
    fprintf(stderr,"init grab_scr %d %d\n",width,height);
    cur_fmt=xaw_to_fmt(cur_f);
    if(cur_fmt==NULL) {
      fprintf(stderr,"format not available by card\n");
      return NULL;
    }
    sizeframe=width*height*cur_fmt->Bpp;
    if(!bktr_yuvbug || cur_f!=VIDEO_YUY2)
      sizeframe2=sizeframe;
    else
      sizeframe2=sizeframe*2/cur_fmt->Bpp;
    if(-1 == ioctl(bttvfd, METEORSACTPIXFMT, cur_fmt))
      perror("ioctl METEORSACTPIXFMT");
    memset(&geom,0,sizeof(geom));
    geom.frames = 1;
    geom.oformat =  xaw_to_geo(cur_f);
    geom.columns=width;
    geom.rows=height;
    if(-1 == ioctl(bttvfd, METEORSETGEO, &geom)) {
      perror("ioctl METEORSETGEO");
      return NULL;
    }
    if(!bktr_read) {
      map = mmap(NULL, sizeframe, PROT_READ, MAP_SHARED, bttvfd, 0);
      if(map==MAP_FAILED) {
	perror("mmap");
	return NULL;
      }
      arg=SIGUSR1;
      if(-1 == ioctl(bttvfd, METEORSSIGNAL, &arg))
	perror("ioctl METEORSSIGNAL grab_scr");
      arg = METEOR_CAP_CONTINOUS ;
      printf("BEGIN CAPTURE\n");
      if(ioctl(bttvfd, METEORCAPTUR, &arg)<0) {
	perror("ioctl METEORCAPTUR CONTINUOUS");
	return NULL;
      }
    }
    if(nbufs>0) buf=realloc(buf,sizeframe2*nbufs);
  }

  if(nbufs>0) {
    img++; if(img==nbufs) img=0;
    if(dest==NULL) dest=&buf[img*sizeframe2];
  } 
  if(dest==NULL) {
    fprintf(stderr,"only works with nbufs>0\n");
    return NULL;
  }
#ifdef _POSIX_THREAD_IS_GNU_PTH
  pthread_yield_np();
#endif
  gettimeofday (&tv, NULL);
  videostampmin = tv.tv_usec/1000000.0+tv.tv_sec;
  if(!bktr_read) {
    sigprocmask(0,NULL,&sa_mask);
    sigsuspend(&sa_mask);
    gettimeofday (&tv, NULL);
    if(bktr_yuvbug && cur_f==VIDEO_YUY2)
      rgb24_to_yuv(dest,map,sizeframe);
    else 
      fast_memcpy(dest,map,sizeframe2);
  } else {
    read(bttvfd, dest, sizeframe);
    gettimeofday (&tv, NULL);
  }
  videostampmax = tv.tv_usec/1000000.0+tv.tv_sec;
  if(bktr_swap ^ (cur_f==VIDEO_YUY2 && !bktr_yuvbug && !cur_fmt->swap_bytes))
     swap_rgb16(dest,sizeframe2/2);
  if(nbufs>0 && dest!=&buf[img*sizeframe2])
    fast_memcpy(&buf[img*sizeframe2],dest,sizeframe2);
  return dest;
}


static void *
grab_scr (void *dest, int width, int height, int single, int dest_xsize)
{
  return bktr_getframe(dest,width,height);
}

static void *
grab_one (int x11_format, int width, int height)
{
  printf("grab_one\n");
  return NULL;
}

static int
grab_tune (unsigned long freq)
{
  if (-1 == ioctl (tunerfd, TVTUNER_SETFREQ, &freq))
    perror ("ioctl TVTUNER_SETFREQ" );
  return 0;
}

static int
grab_tuned ()
{
  return 1;
}

static int
grab_input (int input, int norm)
{
  if(input!=-1)
    if(-1 == ioctl(bttvfd, METEORSINPUT,&inputs_map[input]))
      perror("ioctl METEORSINPUT");
  if(norm!=-1) {
    if(-1 == ioctl(bttvfd, BT848SFMT,&norms_map[norm]))
      perror("ioctl BT848SFMT");
    if(-1 == ioctl(bttvfd, METEORSFMT, &norms_map2[norm]))
      perror("ioctl METEORFMT");
  }
  return 0;
}

static void control(int v,int ioc,int min,int max) {
  int arg=min+v*(max-min)/65535;
  fprintf(stderr,"v=%d arg=%d min=%d max=%d\n",v,arg,min,max);
  if(-1==ioctl(tunerfd,ioc,&arg)) {
    perror("ioctl control");
  }
}

static int
grab_picture (int color, int bright, int hue, int contrast)
{
  if(color!=-1)
    control(color,BT848_SCSAT,BT848_SATUREGMIN,BT848_SATUREGMAX);
  if(bright!=-1)
    control(bright,BT848_SBRIG,BT848_BRIGHTREGMIN,BT848_BRIGHTREGMAX);
  if(hue!=-1)
    control(hue,BT848_SHUE,BT848_HUEREGMIN,BT848_HUEREGMAX);
  if(contrast!=-1)
    control(contrast,BT848_SCONT,BT848_CONTRASTREGMIN,BT848_CONTRASTREGMAX);

  return 0;
}

static int
grab_audio (int mute, int volume, int *mode)
{
  int arg;
  if (mute != -1) {
    arg = mute ? AUDIO_MUTE : AUDIO_UNMUTE;
    if (-1 == ioctl (tunerfd, BT848_SAUDIO, &arg))
      perror ("ioctl BT848_SAUDIO");
  }
  return 0;
}

static int is525(int norm) {
  printf("@@@@@ %d %d %d\n",norm,norms_map2[norm],METEOR_FMT_NTSC);
  return norms_map2[norm]==METEOR_FMT_NTSC;
}

static int issecam(int norm) {
  return norms_map2[norm]==METEOR_FMT_SECAM;
}

static void* get_buf(int i){
  return buf + sizeframe2*i;
}
#endif
